#!/usr/bin/perl

#------------------------------------------------------------------------------
# Program to build the ChiliAsp Apache Web Server Stub for Cobalt platform 
# Only.
# Author : Sriram Natarajan (sn123202)
# Objectives:-
# a. If ChiliAsp module is either Not installed or mod_casp2.c file is not 
#    located make sure that the apache configuration file does NOT have any 
#    ChiliAsp configurations.
# b. Try to build ChiliAsp module if mod_casp2.c file is located in 
#    /home/chiliasp directory.
# c. Use 'apxs' to build in on Cobalt's platforms.
# d. If the build went successfully copy this module to /etc/httpd/modules
#    directory and make sure that the ChiliAsp configurations on Apache conf
#    is updated correctly.
#------------------------------------------------------------------------------

use strict;
use POSIX ;
umask 0;

#==============================================================================
#------------------------ Make sure that the user is a root
#==============================================================================
if ($> != 0) {
  print STDOUT "You will need to be root to execute this script.\n";
  exit 1;
}

# Global scoped variables.

my %chiliCobalt = 
 (
   "webserver_conf" => "/etc/httpd/conf/httpd.conf" ,
   "webserver_srm_conf" => "/etc/httpd/conf/srm.conf",
   "webserver_binary" => "/usr/sbin/httpd",
   "webserver_port" => undef,
   "webserver_root_dir" => undef,
   "webserver_modules_dir" => undef,
   "webserver_version" => undef,
   "asp_home" => "/home/chiliasp" ,
   "asp_installed_db" => "/home/chiliasp/.installed_db" ,
   "asp_modules_dir" => "/home/chiliasp/module/source/build",
   "asp_version" => undef,
   "chili_ini" => "/var/opt/casp/chili.ini",
   "os_type" => "linux2",
   "distribution" => "Cobalt",
   "build_target" => "optimized",
 );

my $pGlobals = \%chiliCobalt;

my $ERROR_LOG = undef;
my $SUMMARY_LOG = undef;
my $REGISTER_LOG = undef;
my $COMPONENT_LOG = undef;

my %CONST_OutputModes =
  (
   Log          => 1,
   Display      => 2,
   Both         => 3
  );

#------------------------------------------------------------------------------
# Collection of Helper routines.
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# Attempt to resolve the location of a binary <name>, using system directories
# before any other.  The optional 2nd, 3rd, etc. arguments specify alternative
# root directories to search.  It should be noted that these directories are
# prepended to the list to be searched and will thus take precedence over
# default system directories.
#
# Note(0): If no executable path can be found, undef is returned.
# Note(1): The result is the full path name of the binary (not merely the
#          directory).
#------------------------------------------------------------------------------
sub locate_binary
{
  ($#_ >= 0) || die "Invalid number of arguments.";

  my ($name,@optional_dirs) = @_;
  my $result = undef;

  if ($#optional_dirs >= 0) {
    foreach(@optional_dirs) {
      my $possibility = "$_/$name";
      if (-x $possibility) {
        return ($possibility) ;
      }
    }
  }

  foreach("/sbin","/usr/sbin","/usr/local/sbin","/bin",
          "/usr/bin","/usr/local/bin","/usr/local/bin")
  {
    my $possibility = "$_/$name";
    if (-x $possibility) {
      $result = $possibility;
    }
  }

  return($result);
}

#------------------------------------------------------------------------------
# Returns the directory name of the specified path (like the Unix dirname
# command).
#
# dirname("/usr/home/deanb") = "/usr/home"
# dirname("/usr/home/deanb/") = "/usr/home"
# dirname("foobar") = "."
# dirname("foobar/fubar") = "foobar"
#------------------------------------------------------------------------------
sub dirname
{
  my @result = @_;

  foreach(@result) {
    s@/$@@;

    if (m@/@) {
      s@/[^/]*$@@;
    }
    else {
      $_ = ".";
    }
  }

  wantarray ? return(@result) : return($result[0]);
}

#------------------------------------------------------------------------------
# Returns the base filename of the specified path (like the Unix basename
# command).
#
# basename("/pub/home/deanb") = "deanb"
# basename("/pub/home/deanb/") = "deanb"
# basename("foobar") = "foobar"
# basename("foobar/fubar") = "fubar"
#------------------------------------------------------------------------------
sub basename
{
  my @result = @_;

  foreach(@result) {
    s@/$@@;

    if (m@/([^/]*)$@) {
      $_ = $1;
    }
  }

  wantarray ? return(@result) : return($result[0]);
}

#------------------------------------------------------------------------------
# Strip the extension from the specified path.
#------------------------------------------------------------------------------
sub strip_extension
{
  ($#_ == 0) || die "Invalid number of arguments.";

  my ($path) = @_;
  $path =~ s/\.[^\.]*$//;

  return($path);
}

#------------------------------------------------------------------------------
# Find <find_count> file(s) under the specified <root> directory that matches
# <filename>.  If <find_count> is left unspecified (or is undef), it is
# interpreted to mean, find all.
#
# Upon failure, an empty list is returned.
#------------------------------------------------------------------------------
sub find_files
{
  (($#_ == 1) || ($#_ == 2)) || die "Invalid number of arguments.";

  my ($root,$filename,$find_count) = @_;
  my @result;

  if (not defined $find_count) {
    $find_count = 2**31;
  }

  my @subdirs = ($root);
  my $subdir;

  while($subdir = pop @subdirs) {
    local *SUBDIR;
    opendir(SUBDIR,$subdir) || die "Invalid directory: $subdir";
    my @files = readdir(SUBDIR);
    closedir(SUBDIR);

    foreach(@files) {
      (($_ ne ".") && ($_ ne "..")) || next;

      $find_count || return(@result);
      my $path = "$subdir/$_";

      if ($_ eq $filename) {
        push @result, $path;
        $find_count--;
      }

      if ((not -l $path) && (-d $path)) {
        push @subdirs, $path;
      }
    }
  }

  wantarray ? return(@result) : return($result[0]);
}

sub find_file
{
  ($#_ == 1) || die "Invalid number of arguments.";

  my @result = find_files(@_,1);

  wantarray ? return(@result) : return($result[0]);
}

#------------------------------------------------------------------------------
# Copy <source> to <destination>.  Upon success, 1 is returned.  Otherwise,
# undef is returned.
#------------------------------------------------------------------------------
sub copy
{
  (@_ == 2) || die "Invalid number of arguments.";
  my ($source,$destination) = @_;

  unless (-f "$source" || -d "$source" || -l "$source") {
    error ("Source - $source does NOT exist.");
    return (undef);
  }
  unless (-f "$destination" || -d "$destination" || -l "$destination") {
    error ("Target - $destination does NOT exist.");
    return (undef);
  }
  unless(system("/bin/cp -f $source $destination")) {
    return(1);
  }

  return(undef);
}

#------------------------------------------------------------------------------
# Functions takes a <header> and a list of string which it will concatenate
# together as follows:
# <header><string1>
#         <string2>
#            ...
#         <stringN>
#------------------------------------------------------------------------------

sub headerize
{
  my $header = shift @_;
  my $separator = "\n";
  $separator .= " " x length($header);
  return($header . join($separator,@_) . "\n");
}

#------------------------------------------------------------------------------
# Function takes a pointer to caller info structure (containing a list of
# package, filename, and line number), a header, then the subsequent lines
# of output.
#------------------------------------------------------------------------------
my $report_internal = sub
{
  my $pCaller_info = shift @_;
  my $output_mode = shift @_;
  my $return_type = shift @_;
  my $result = headerize(@_);

  if ($CONST_OutputModes{Display} & $output_mode) {
    print STDERR $result;
  }

  if (($CONST_OutputModes{Log} & $output_mode) && defined $ERROR_LOG) {
    my $time = localtime(time);
    my ($package,$filename,$line) = @$pCaller_info;

    print $ERROR_LOG ("====================================================== $time\n",
                      $result,
                      "Encountered at:\n",
                      "  Package: $package\n",
                      "  File: $filename\n",
                      "  Line: $line\n",
                      "===============================================================================\n"
                     );
  }

  if ($return_type) {
    # Wants an array.
    return();
  }
  else {
    return(undef);
  }
};

#------------------------------------------------------------------------------
# The report functions take a header, followed by a list of strings to display
# on separate lines past the header:
#
# Example:
# report("Warning: ",
#        "Continuing on with the current course will result in the",
#        "destruction of your server.");
#
#  ==> "Warning: Continuing on with the current course will result in the
#                destruction of your server."
#
# Note: report displays only to the screen
#       report_log displays only to the log
#       report_both displays to both the log and the screen
#------------------------------------------------------------------------------
sub report
{
  return(&$report_internal([caller],$CONST_OutputModes{Display},wantarray,@_));
}

sub report_log
{
  return(&$report_internal([caller],$CONST_OutputModes{Log},wantarray,@_));
}

sub report_both
{
  return(&$report_internal([caller],$CONST_OutputModes{Both},wantarray,@_));
}

#------------------------------------------------------------------------------
# The error function(s) works similarly to the report function except that the
# header is implicitly "Error: ".
#
# Note: error displays to both the log and the screen
#       error_log displays to only the log
#       error_display displays to only the screen
#------------------------------------------------------------------------------
sub error
{
  return(&$report_internal([caller],$CONST_OutputModes{Both},wantarray,"Error: ",@_));
}

sub error_log
{
  return(&$report_internal([caller],$CONST_OutputModes{Log},wantarray,"Error: ",@_));
}

sub error_display
{
  return(&$report_internal([caller],$CONST_OutputModes{Display},wantarray,"Error: ",@_));
}

#------------------------------------------------------------------------------
# Encode newline characters into \, and \ into \\.
#------------------------------------------------------------------------------
sub encode
{
  ($#_ == 0) || die "Invalid number of arguments.";

  my ($value) = @_;
  $value =~ s@\\@\\\\@;
  $value =~ s@\n@\\@;
  return($value);
}

#------------------------------------------------------------------------------
# Encode \\ into \, and \ into a newline.
#------------------------------------------------------------------------------
sub decode
{
  ($#_ == 0) || die "Invalid number of arguments.";

  my ($value) = @_;
  $value =~ s@([^\\])\\@$1\n@;
  $value =~ s@\\\\@\\@;
  return($value);
}

#------------------------------------------------------------------------------
# Open an ini file, returning a reference to an internally maintained structure
# which represents the ini file.  The specified ini file need not exist.  If
# it does not exist, a new ini file will be created.
#------------------------------------------------------------------------------
sub openIni
{
  ($#_ == 0) || die "Error: Invalid number of parameters (must be 1).";
  my ($fileName) = @_;

  if (stat($fileName)) {
    return(load($fileName));
  }
  elsif (open(FILE,">>$fileName")) {
    chmod(0644,$fileName);
    close(FILE);
    return(load($fileName));
  }
  else {
    return(error("Unable to open $fileName for writing - open.\nReason: $!"));
  }
}

#------------------------------------------------------------------------------
# Opens an existing ini file.  If the file does not exist, an error will be
# returned.  Otherwise, a reference to an ini structure will be returned.
# NOTE: The functionality of load is identical to open, except for the
# requirement of the ini file to already exist.
#------------------------------------------------------------------------------
sub load
{
  ($#_ == 0) || die "Error: Invalid number of parameters (must be 1).";

  my ($fileName) = @_;

  unless (open(FILE,"<$fileName")) {
    return(error("Unable to open $fileName for reading - load\nReason: $!."));
  }
  else {
    my $line;
    my %result;

    my %newSection;
    my $newName;
    my @section_order;
    my %comments;

    while($line = <FILE>) {
      if ($line =~ /^\s.*$/) {
        # Ignore empty lines.
        next;
      }
      elsif ($line =~ /^\s*\#\s*(.*)$/) {
        if ($newName) {
          push @{$comments{$newName}},decode($1);
        }
        else {
          # Global comment name = [
          push @{$comments{"["}},decode($1);
        }
        next;
      }
      elsif ($line =~ /^\s*\[([^\]]*)\]\s*$/) {
        if ($newName) {
          my %insert=%newSection;
          $result{$newName} = \%insert;
          $newName = "";
          %newSection = ();
        }

        $newName = $1;
        push @section_order, $newName;
      }
      elsif ($line =~ /^\s*([^=]+)\s*=\s*(.*)$/) {
        my $key = decode($1);
        my $value = decode($2);

        # For backware compatibility reasons, if there is a double-quote at
        # the head and tail of a value, strip them.
        $value =~ s@^\"(.*)\"$@$1@;
        $newSection{$key} = $value;
      }
      elsif ($line =~ /^\s*([^=]+)\s*$/) {
        $newSection{$1} = "";
      }
      else {
        note("Skipping garbage line: $line");
      }
    }

    if ($newName) {
      $result{$newName} = \%newSection;
    }

    $result{"]section_order"} = \@section_order;
    $result{"]comments"} = \%comments;

    close(FILE);
    return \%result;
  }
}

#------------------------------------------------------------------------------
# Saves the information contained in the specified passed ini reference into
# the specified file.
#------------------------------------------------------------------------------
sub save
{
  ($#_ == 1) || die "Error: Invalid number of parameters (must be 2).";

  my ($pIniData,$fileName) = @_;
  my %iniData = %{$pIniData};

  unless (open(FILE,">$fileName")) {
    return(error("Unable to open $fileName for writing - save.\nReason: $!"));
  }
  else {
    chmod(0644,$fileName);

    # Handle external comments.
    my %comments;

    if (exists $iniData{"]comments"}) {
      %comments = %{$iniData{"]comments"}};
    }

    if (exists $comments{"["}) {
      my @external_comments = @{$comments{"["}};

      foreach(@external_comments) {
        print FILE "# $_\n";
      }

      print FILE "\n";
    }

    my @section_order;

    if (exists $iniData{"]section_order"}) {
      @section_order = @{$iniData{"]section_order"}};
    }
    else {
      @section_order = getSectionNames($pIniData);
    }

    # For each section, print it out.
    foreach (@section_order) {
      my $section_name = $_;

      print FILE "[$section_name]\n";

      my %records = %{$iniData{$_}};
      my @recordKeys = map(encode($_),keys %records);

      foreach (@recordKeys) {
        my $record = encode($records{$_});
        print FILE "$_=$record\n";
      }

      # Handle internal comments.
      if (exists $comments{$section_name}) {
        my @internal_comments = map(encode($_),@{$comments{$section_name}});

        foreach(@internal_comments) {
          print FILE "# $_\n";
        }
      }

      print FILE "\n";
    }

    close(FILE);
    return(1);
  }
}

#------------------------------------------------------------------------------
# Insert a new section into the passed ini structure.  If the named section
# already exists, the return value will be that section.  It is thus impossible
# to store more than one section of the same name in an ini structure.  Upon
# success, the return value will be a reference to a section structure (a hash).
# It should be noted that changes made to this structure will result in
# alterations to the passed ini structure (by design).
#------------------------------------------------------------------------------
sub insertSection
{
  ($#_ == 1) || die "Error: Invalid number of parameters (must be 2).";

  my ($pIniData,$section) = @_;

  if (exists $pIniData->{$section}) {
    return(openSection($pIniData,$section));
  }
  else {
    my %newSection;
    $pIniData->{$section} = \%newSection;

    if (not grep { $section eq $_ } @{$pIniData->{"]section_order"}}) {
      push @{$pIniData->{"]section_order"}}, $section;
    }

    return(\%newSection);
  }
}

#------------------------------------------------------------------------------
# Delete the specified section from the passed ini structure reference. The
# reutnr value is undef, if the operation failed, or non-undef on success.  It
# should be noted that the passed ini reference (or rather the ini structure to
# which that refers), will be altered via this method.
#------------------------------------------------------------------------------
sub deleteSection
{
  ($#_ == 1) || die "Error: Invalid number of parameters (must be 2).";

  my ($pIniData,$section) = @_;

  if (exists $pIniData->{$section}) {
    my $pRemoved = $pIniData->{$section};
    delete $pIniData->{$section};

    if (grep { $section eq $_ } @{$pIniData->{"]section_order"}}) {
      my @newOrder;
      foreach(@{$pIniData->{"]section_order"}}) {
        if ($section ne $_) {
          push @newOrder, $_;
        }
      }

      $pIniData->{"]section_order"} = \@newOrder;
    }

    return($pRemoved);
  }
  else {
    return(undef);
  }
}

#------------------------------------------------------------------------------
# Retrieve a list of names from the passed ini structure reference.
#------------------------------------------------------------------------------
sub getSectionNames
{
  ($#_ == 0) || die "Error: Invalid number of parameters (must be 1).";

  my ($pIniData) = @_;
  my %iniData = %{$pIniData};

  my @result;
  foreach(keys %iniData) {
    # Strip special fields.
    if (not /\]/) {
      push @result,$_;
    }
  }

  return(@result);
}

#------------------------------------------------------------------------------
# Open the specified section if the section exists.  If the section does not
# exist failure will be returned.  This method is identical to insertSection,
# except that if the section does not already exist, failure will be returned.
#------------------------------------------------------------------------------
sub openSection
{
  ($#_ == 1) || die "Error: Invalid number of parameters (must be 2).";

  my ($pIniData,$section) = @_;

  if (exists $pIniData->{$section}) {
    return($pIniData->{$section});
  }
  else {
    return(undef);
  }
}

#------------------------------------------------------------------------------
# Each section also has associated with it a comments section.  This section
# of comments will be appended to the end of the section data as # <comment>
# ... for each comment contain in the list.  When read from the ini file in
# question, they will be output in the order read in.  This method returns a
# reference to an array.  In this way, by changing this array, you will be
# changing the ini data.  If the section named does not already exist, undef
# will be returned (see insertComments).
#------------------------------------------------------------------------------
sub openComments
{
  ($#_ == 0) || ($#_ == 1) || die "Invalid number of arguments.";

  my ($pIniData,$section) = @_;

  if ((not $section) || ($section eq "[")) {
    $section = "[";
  }
  elsif (not exists $pIniData->{$section}) {
    return(undef);
  }

  if (not exists $pIniData->{"]comments"}) {
    my %newComment;
    $pIniData->{"]comments"} = \%newComment;
  }

  if (not exists $pIniData->{"]comments"}->{$section}) {
    my @newComments;
    $pIniData->{"]comments"}->{$section} = \@newComments;
  }

  return($pIniData->{"]comments"}->{$section});
}

#------------------------------------------------------------------------------
# Identical to openComments except that if the specified section does not exist
# it will be added.
#------------------------------------------------------------------------------
sub insertComments
{
  ($#_ == 0) || ($#_ == 1) || die "Invalid number of arguments.";

  my ($pIniData,$section) = @_;

  if (not $section) {
    $section = "[";
  }
  elsif (not exists $pIniData->{$section}) {
    my %newSection;
    $pIniData->{$section} = \%newSection;
  }

  return(openComments($pIniData,$section));
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# External functions
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#------------------------------------------------------------------------------
# Checks whether the Web Server is installed on the customer machine.
# If the web server is done, it does a basic checking to ensure it is a 
# valid Apache/Web Server installation on the client machine and then goes 
# to fill in the required values in its hash table.
# returns 1 on success and undef on error.
#------------------------------------------------------------------------------
sub isWebServerInstalled
{
  sub isApacheInstalled
  {
    my ($httpd_conf,$httpd_binary) = (@_) ;

    # we have no reliable way of looking for the httpd.conf file.
    # try looking for the apache configuration file at the standard
    # location and with these informations we need to determine the type.

    detectWebServer($httpd_conf,$httpd_binary) ;
    if ($pGlobals->{'webserver_type'} !~ /apache/i) {
      error ("This web server does NOT seems to be of Type Apache.");
      return (undef);
    }
    return (1);
  }

  # Start of routine to detect whether the web server is installed or not.
  # for now, we will just look for the webserver of type apache.
  return (isApacheInstalled(@_)) ;
}

#------------------------------------------------------------------------------
# This routine is responsible for detecting web server type and thereby extracting 
# appropriate informations about the configuration of this web server.
# If the configuration file is specified, it uses this file to look for the 
# configuration information. If the configuration file is not found, then
# we basically look for the configuration file in the standard locations.
# returns 1 on success and undef on error.
#------------------------------------------------------------------------------
sub detectWebServer
{
  sub detectApache
  {
    # detect and locate the apache configuration and binary location 
    # information.
    my ($httpd_conf,$httpd_binary) = (@_);
    my @httpd_conf_locations =
    ( 
     "/etc/httpd/conf/httpd.conf",
     "/var/lib/apache/conf/httpd.conf",
     "/usr/local/apache/conf/httpd.conf",
     "/usr/local/chilisoft/apache/conf/httpd.conf",
    );
    my @httpd_binary_locations =
    (
     "/usr/sbin/httpd",
     "/usr/sbin/httpsd",
     "/var/lib/apache/sbin/httpd",
     "/usr/local/apache/bin/httpd",
     "/usr/local/chilisoft/apache/bin/httpd",
    );

    unless (-f "$httpd_conf") {
      foreach $httpd_conf (@httpd_conf_locations) {
        last if (-f "$httpd_conf");
      }
    }
    unless (-f "$httpd_binary") {
      foreach $httpd_binary (@httpd_binary_locations) {
        last if (-f "$httpd_binary" );
      }
      $httpd_binary = locate_binary("httpd") if (! -f "$httpd_binary") ;
    }

    # now let us parse the configuration file and try to load the informations
    # into the global hash table.
    unless (open(CONFIG,"<$httpd_conf")) {
      error("Unable to open the httpd configuration file.");
      error("Error occurred while trying to detect the Web Server Type.");
      close(CONFIG);
      return (undef);
    }
    while (<CONFIG>) {
      if (/^\s*ServerRoot\s+(\S+)/) {
        $pGlobals->{webserver_root_dir} = $1;
      }
      if (/^\s*Port\s+([0-9]+)/) {
        $pGlobals->{webserver_port} = $1;
      }
      last if ($pGlobals->{webserver_root_dir} && $pGlobals->{webserver_port}); 
    }
    close(CONFIG);

    if (! -x "$httpd_binary") {
      foreach("sbin","bin","src") {
        # Check for secure server first.
        my $subdir = $_;
        foreach ("httpsd","httpd","apache","apached") {
          if (-r "$pGlobals->{webserver_root_dir}/$subdir/$_") {
	    $httpd_binary = "$pGlobals->{webserver_root_dir}/$subdir/$_";
	    last;
	  }
        }
      }
    }
    my $version_string = qx($httpd_binary -v 2>/dev/null);
    my $define_string = qx($httpd_binary -V 2>/dev/null);
    my $module_string = qx($httpd_binary -l 2>/dev/null);

    if ($version_string =~ m@Server version:.*/(\d+)\.(\d+)\.(\d+)@s) {
      $pGlobals->{webserver_version} = "$1.$2.$3" ;
    } else {
      error("Unable to find a Apache Web Server version information.");
      return (undef);
    }
    
    if ($version_string !~ /apache/i) {
      error("Unable to find a valid Apache Web Server .");
      return (undef);
    } else {
      $pGlobals->{webserver_type} = "apache" ;
    }

    ($define_string =~ /EAPI/i) ? ($pGlobals->{webserver_api} = "eapi") 
                                : ($pGlobals->{webserver_api} = "standard") ;
    ($module_string =~ /mod_so/i) ? ($pGlobals->{webserver_dso} = "yes") 
                                : ($pGlobals->{webserver_dso} = "no") ;
    
    # now find out the apache modules directory and the support configuration file.
    if (( -l "$pGlobals->{webserver_root_dir}/modules") ||
        ( -d "$pGlobals->{webserver_root_dir}/modules")) 
    {
      $pGlobals->{webserver_modules_dir} = "$pGlobals->{webserver_root_dir}/modules" ; 
    }
    if ( -f "$pGlobals->{webserver_root_dir}/conf/srm.conf") {
      $pGlobals->{webserver_srm_conf} = "$pGlobals->{webserver_root_dir}/conf/srm.conf" ; 
    }
    return (1);
  }

  # Start of routine to detect the web server type and cache the webserver
  # information.
  return (detectApache(@_));
}

#------------------------------------------------------------------------------
# This routine is responsible for removing the ASP configuration from the
# webserver configuration file.
#------------------------------------------------------------------------------
sub remove_WebServerCAspConfig
{
  sub remove_ApacheCAspConfig
  {
    ($#_ == 0) or return (error("Error in remove_ApacheCAspConfig:Insufficient Input parameters."));

    my ($conf_file) = (@_);
    my $support_conf_file ;
    unless (-f $conf_file) {
      error ("Input configuration file does not exist. Unable to remove the ASP Configurations.");
      return (undef);
    }
    unless (open(CONFIG,"<$conf_file")) {
      error ("Error occurred in remove ASP Configurations: Unable to open the Configuration file.");
      return (undef);
    }
    if ($conf_file eq $pGlobals->{webserver_conf}) {
      if (exists $pGlobals->{webserver_srm_conf} && -f "$pGlobals->{webserver_srm_conf}") {
        $support_conf_file = $pGlobals->{webserver_srm_conf} ;
      }
    }
    unless ($support_conf_file) {
      while (<CONFIG>) {
        if (/^\s*ServerRoot\s+(\S+)/) {
          my $root = $1;
          if (-d "$root" && -f "$root/conf/srm.conf") {
            $support_conf_file = "$root/conf/srm.conf" ;
          }
          last ;
        }
      }
    }
    close(CONFIG);

    # Now start with the ChiliAsp remove configuration code.
    my @conf_files ;
    push @conf_files,$conf_file if (-f "$conf_file") ;
    push @conf_files,$support_conf_file if (-f "$support_conf_file") ;

    foreach my $conf (@conf_files) {
      my $dupname = "$conf.pre-chiliasp-upgrade" ;
      unlink("$dupname");
      system("/bin/cp $conf $dupname");
      unless (open(IN,"<$dupname") && open(OUT,">$conf")) {
        error("Unable to open $conf for modification.");
        return (undef);
      }
      ########## Strip any Chili!Soft ASP line from the httpd.conf file.
      ########## Keep track of the PidFile directive.
      while (<IN>) {
        print OUT if ((! /AddHandler.*\.as[pa]|CaspLib|Chili!Soft ASP|mod_casp2|casp2_module/i));
      }
      close(IN);
      close(OUT);
    }
    return (1);
  }

  # start of routine to rmeove the webserver-asp association.
  return (remove_ApacheCAspConfig(@_));
}

#------------------------------------------------------------------------------
# This routine is responsible for configuring the WebServer with ChiliAsp 
# configuration informations.
# Perform the job of apxs. If a fourth argument value (which evaluates to true),
# does not exist, the default semantics of apxs are used.  That is, the new
# module is appended to the end of the AddModule / LoadModule lists.  
# Otherwise, if it does evaluate to true, the module will be inserted at the
# head of the list (ie. prepended).
#
# It needs the webserver configuration file as its input.
# returns undef on error and 1 on success.
#------------------------------------------------------------------------------
sub enable_WebServerCAspConfig
{
  sub enable_ApacheCAspConfig
  {
    (($#_ == 1) || ($#_ == 2)) || die "Invalid number of arguments.";

    my ($conf,$entry_point,$module_name,$prepend) = @_;

    unless (-f $module_name)  {
      error("Specified module '$module_name' does not exist.");
	  return (undef);
    }
    unless (-f $conf) {
      error ("Input configuration file does not exist. Unable to remove the ASP Configurations.");
      return (undef);
    }

    my $dupname = "$conf.pre-chiliasp-apxs";
    unlink($dupname);
    system("/bin/cp $conf $dupname");
    unless(open(SRC,"<$dupname") && open(HTTPD,">$conf")) 
    {
      # Back out any changes.
      close(SRC);
      close(HTTPD);
      system("/bin/cp -f $dupname $conf");
      error("Unable to open $conf for modification.");
	  return (undef);
    }

    my $module_root = strip_extension(basename($module_name));
    my $quoted_module_root = quotemeta($module_root);
    my @lines ;

    foreach(<SRC>) {
      if ((not m@[/\s]$quoted_module_root@) && (not /Chili!Soft/) && (not /$entry_point/)) {
        push @lines,$_;
      }
    }

    my @result;
    my $line_count = $#lines;
    if (grep { /^\s*ClearModuleList/ } @lines) {
      ### First find the ClearModuleList line (or actually the first 
      ### non-comment-line, before that).
      my $ClearModule_index;

      # Accomodate SuSE bug of duplicating the LoadModule / AddModule section in
      # the httpd.conf file.  We make sure we add ourself around the last 
      # ClearModuleList, rather than the first.
      for (my $i=$line_count; $i >= 0; $i--) {
        if ($lines[$i] =~ /^\s*ClearModuleList/) {
          $ClearModule_index = $i;
          last;
        }
      }
      # Search for first previous non-comment line.
      for (my $i = $ClearModule_index - 1; $i >= 0; $i--) {
        if ($lines[$i] !~ /^\s*\#/) {
          $ClearModule_index = $i;
          last;
        }
      }

      ### Then find the last AddModule index (or actually the first
      ### non-comment-line, after that).
      my $last_AddModule_index;
      for (my $i=$line_count; $i >= 0; $i--) {
        if ($lines[$i] =~ /^\s*AddModule/) {
          $last_AddModule_index = $i;
          last;
        }
      }
      if (not $last_AddModule_index) {
        $last_AddModule_index = $ClearModule_index + 1;
      }

      # Discover if we are in a <IfDefine> block
      my $insideIfDefine = 0;
      for (my $i = 0; $i <= $last_AddModule_index; $i++) {
        if ($lines[$i] =~ m@^\s*<IfDefine@) {
          $insideIfDefine++;
        }
        elsif ($lines[$i] =~ m@^\s*</IfDefine@) {
          $insideIfDefine--;
        }
      }
      if ($insideIfDefine > 0) {
        # We will make sure to add ourself after the appropriate closing </IfDefine>
        # (I certainly hope their httpd.conf is valid).
        for (my $i = $last_AddModule_index + 1; ($i <= $line_count); $i++) 
        {
          if ($lines[$i] =~ m@^\s*<IfDefine@) {
            $insideIfDefine++;
          }
          elsif ($lines[$i] =~ m@^\s*</IfDefine@) {
            $insideIfDefine--;
          }
          if ($insideIfDefine <= 0) {
            $last_AddModule_index = $i + 1;
            last;
          }
        }
      }
      # Find first non-comment line.
      for (my $i = $last_AddModule_index + 1; $i <= $line_count; $i++) {
        if ($lines[$i] !~ /^\s*\#/) {
          $last_AddModule_index = $i;
          last;
        }
      }

      # Now glue it all together.
      @lines = (
        @lines[0 .. ($ClearModule_index - 1)],
        "#-------------------------------------------------- Chili!Soft ASP\n",
        "LoadModule $entry_point\t$module_name\n",
        "#-------------------------------------------------- Chili!Soft ASP\n",
        @lines[$ClearModule_index .. ($last_AddModule_index - 1)],
        "#-------------------------------------------------- Chili!Soft ASP\n",
        "AddModule $module_root.c\n",
        "#-------------------------------------------------- Chili!Soft ASP\n",
        @lines[$last_AddModule_index .. $line_count]
      );
    }
    else {
      if (grep { /^\s*LoadModule/ } @lines) {
        my $first = 1;
        @lines = reverse map { 
          if ($first && /^\s*LoadModule/) {
            $first = 0;
            (	
             "#-------------------------------------------------- Chili!Soft ASP\n",
             "LoadModule $entry_point\t$module_name\n",
             "#-------------------------------------------------- Chili!Soft ASP\n",
             $_
            )
          }
          else {
            $_;
          }
        } reverse @lines;
      }
      else {
        # Put the LoadModule at the top of the file.
        @lines = (
        "#-------------------------------------------------- Chili!Soft ASP\n",
        "LoadModule $entry_point\t$module_name\n",
        "#-------------------------------------------------- Chili!Soft ASP\n",
        @lines );
      }
    }
    print HTTPD @lines;

    close(SRC);
    close(HTTPD);

    # It is now safe to wipe the slate clean.
    unlink($dupname);

    return(1);
  }

  # start of routine enable_WebServerCAspConfig.
  return (enable_ApacheCAspConfig(@_));
}

#------------------------------------------------------------------------------
# Checks whether the ASP is installed on the customer machine.
# It either looks for a valid installation on the customer machine by rpm
# query process and if it fails it tries to look for the valid installation
# at /home/chiliasp location.
#------------------------------------------------------------------------------
sub isCAspInstalled  
{
  # In future we should look for a valid ASP Installation in the 
  # /var/opt/casp/chili.ini file.
  # for now, if chiliasp is not installed in /home/chiliasp location,
  # let us bail out.

  unless ( -d "$pGlobals->{asp_home}" ) {
    error ("No Valid ChiliAsp installation is found.");
    return (undef) ;
  }
  unless ( -f "$pGlobals->{asp_installed_db}" ) {
    error ("No valid Installation Registry file is found.");
    return (undef);
  }
  unless ( -d "$pGlobals->{asp_modules_dir}" ) {
    error ("Unable to find the ChiliAsp webserver module stub files.");
    return (undef);
  }
  my $pIni = openIni("$pGlobals->{asp_installed_db}");
  unless ($pIni) {
    error ( "Unable to open the ChiliAsp Installed Database file.");
    return (undef);
  }
  foreach my $local_server (getSectionNames($pIni)) 
  {
    # for now, just look for a valid directory and get out with this
    # simple validation.
    unless ( -d $local_server ) {
      error ("No valid Server Information is found.");
      return (undef);
    }
    my $pSection = openSection($pIni,$local_server);
    $pGlobals->{webserver_conf} = $pSection->{webserver_conf} if (-f "$pSection->{webserver_conf}") ;
    $pGlobals->{webserver_binary} = $pSection->{webserver_binary} if (-f "$pSection->{webserver_binary}") ;
  }

  # Make sure we have a valid ASP Installation in this location.
  return (1);
}

#------------------------------------------------------------------------------
# This routine is responsible for building a ChiliAsp Web server stub module.
# To successfully build a ChiliAsp module, this routine depends on the apxs 
# binary to successfully build a ChiliAsp module on the fly. 
# returns undef if unable to build the module successfully.
#------------------------------------------------------------------------------
sub buildCAspModule 
{
  # make sure we have ChiliAsp installed and the mod_casp2.c source file. 
  unless (isCAspInstalled) {
    error ("Unable to find a valid ChiliAsp installation to build a web server module.");
    return (undef);
  }

  # now load the web server configuration file used by this ChiliAsp installation.
  my $webserver_conf; 
  my $webserver_binary ;
  if (exists $pGlobals->{webserver_conf} && -f "$pGlobals->{webserver_conf}") {
    $webserver_conf = $pGlobals->{webserver_conf} ;
  }
  if (exists $pGlobals->{webserver_binary} && -f "$pGlobals->{webserver_binary}") {
    $webserver_binary = $pGlobals->{webserver_binary} ;
  }

  # make sure we have Cobalt's Apache Web Server has been installed and the header files are present.
  # also determine the version of apache.
  unless (isWebServerInstalled($webserver_conf,$webserver_binary)) {
    error ("Unable to find a valid Web Server installation to build a web server module");
    return (undef);
  }

  my $asp_home = $pGlobals->{asp_home} ;
  my $distribution = $pGlobals->{distribution};
  my $build_target = "$pGlobals->{os_type}"."_"."$pGlobals->{build_target}";
  my $chili_module_type = "$pGlobals->{webserver_type}"."_"."$pGlobals->{webserver_version}";
  my $cobalt_modules_dir = "$asp_home/module/$build_target/$chili_module_type/non-standard/$distribution" ;
  my $standard_modules_dir = "$asp_home/module/$build_target/$chili_module_type/$pGlobals->{webserver_api}" ;
  my $make_build = undef ;
  my $apxs_build_result = undef ;
  my $module_name = undef ;

  if (! -d $cobalt_modules_dir) {
    my $mkdir_bin = locate_binary("mkdir") ;
    system("$mkdir_bin -p $cobalt_modules_dir 2>/dev/null") ;
  }

  # make sure we have apxs executable found.
  my $apxs_binary = locate_binary("apxs") ;
  if (-x "$apxs_binary" && locate_binary('cc')) {
    $make_build = 1;
  } else {
    error ("Unable to find either apxs or 'cc' to build the ChiliAsp module.");
    $make_build = 0;
  }

  # try building the module and if successfully built it copy it to the
  # modules directory.
  # if failed, return undef so that the caller can do the error handling 
  # mechanism.
  # now time to start looking for the webserver modules to build.
  unless ( (-d "$pGlobals->{asp_modules_dir}" ) || 
           (-f "$pGlobals->{asp_modules_dir}/mod_casp2.c") || 
           (-f "$pGlobals->{asp_modules_dir}/dispint.h") ) {
    error ("Unable to find either ChiliAsp modules directory or source file to build a web server module.");
    $make_build = 0;
  }

  if ($make_build) {
    my $cwd = getcwd() ;
    chdir $pGlobals->{asp_modules_dir} ;
    unlink("$pGlobals->{asp_modules_dir}/mod_casp2.so") ; 
    $apxs_build_result = system ("$apxs_binary -c -o $pGlobals->{asp_modules_dir}/mod_casp2.so $pGlobals->{asp_modules_dir}/mod_casp2.c 2>/dev/null");
    chdir ($cwd);
    unless ($apxs_build_result) {
      # we were able to build successfully.
      copy ("$pGlobals->{asp_modules_dir}/mod_casp2.so","$cobalt_modules_dir" );
      copy ("$pGlobals->{asp_modules_dir}/mod_casp2.so","$pGlobals->{webserver_modules_dir}");
	  $module_name = "$cobalt_modules_dir/mod_casp2.so" ;
    }
  }

  if (! $make_build || $apxs_build_result || ! -f "$pGlobals->{asp_modules_dir}/mod_casp2.so") 
  {
    # now try to reuse a existing a previous module for this.
    if (-f "$cobalt_modules_dir/mod_casp2.so") {
      error ("apxs returned with error. So re-using a previously built (cobalt) ChiliAsp module for this web server.");
      copy ("$cobalt_modules_dir/mod_casp2.so","$pGlobals->{webserver_modules_dir}");
	  $module_name = "$cobalt_modules_dir/mod_casp2.so" ;
    }
    elsif (-f "$standard_modules_dir/mod_casp2.so") {
      error ("apxs returned with error. So re-using a previously built (generic) ChiliAsp module for this web server.");
      copy ("$standard_modules_dir/mod_casp2.so","$cobalt_modules_dir" );
      copy ("$standard_modules_dir/mod_casp2.so","$pGlobals->{webserver_modules_dir}");
	  $module_name = "$standard_modules_dir/mod_casp2.so" ;
    }
    else {
      # we were not able to successfully built the apache module.
      error ("apxs returned with error. Unable to build the ChiliAsp module");
      remove_WebServerCAspConfig("$pGlobals->{webserver_conf}");
      return (undef);
    }
  }
# Sam: 10/28/02
# No need to modify httpd.conf as Sausalito manages it
#  if (-f "$module_name") {
#    enable_WebServerCAspConfig($pGlobals->{webserver_conf},"casp2_module",$module_name);
#  }
#  dN:6/28/02: this should return something other than undef if it
#  is successful!!!
#  return (undef);
  return 1;
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Start of Main Routine.
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#------------------------------------------------------------------------------
# This routine is the main routine. it is responsible for determining a valid
# ChiliAsp and apache installation on the customer platform. 
# if we did manage to find a valid Installations, then we will look for the 
# apxs binary and cc executable to make sure that we can build a new ChiliAsp
# module on the customer platform.
# If all goes on well then we copy this new module in the web server location
# and finally we are done. 
# if something goes wrong with this sequence, then we will now start the error
# handling process. In the error handling sequence, first make sure we have the
# EAPI version of Apache web server module already on the customer platform. 
# if such module exists copy this module onto the customer platform.
# If no such module exists, then we will now delete the ChiliAsp-WebServer association
#------------------------------------------------------------------------------
{
  unless (buildCAspModule) {
    exit (1);
  }
  report ("ChiliAsp Module successfully installed.");
  exit (0);
}
